//==============================================================================
//
// Title:       3456Terminal
// Purpose:     A short description of the application.
//
// Created on:  2017/9/16 at 23:01:32 by .
// Copyright:   . All Rights Reserved.
//
//==============================================================================

//==============================================================================
// Include files

#include <ansi_c.h>
#include <cvirte.h>     
#include <userint.h>
#include <visa.h>
#include "windows.h"
#include "3456Terminal.h"
#include "toolbox.h"

//==============================================================================
// Constants

//==============================================================================
// Types

//==============================================================================
// Static global variables

static int panelHandle;

//==============================================================================
// Static functions

//==============================================================================
// Global variables

ViStatus status; /* For checking errors */
ViSession defaultRM, instr; /* Communication channels */
ViUInt32 retCount; /* Return count from string I/O */
ViChar buffer[200]; /* Buffer for string I/O */

ViFindList fList;
ViUInt32 numInstrs;
ViChar desc[VI_FIND_BUFLEN];

int quit = 0;
int connected = 0;
int GPIBLock = 0; /* Lock for GPIB bus, to aviod access to GPIB bus at the same time in different thread */
int pointCounter = 0;
double startTime = 0;

//==============================================================================
// Global functions

void enumInstruments()
{
	ClearListCtrl(panelHandle, PANEL_MENU_DEVICE);
	if(viFindRsrc(defaultRM, "GPIB?*INSTR", &fList, &numInstrs, desc) < VI_SUCCESS)
	{
		MessagePopup("Error", "No GPIB device available"); 
		return;
	}
	while(numInstrs--)
	{
		InsertListItem(panelHandle, PANEL_MENU_DEVICE, -1, desc, desc);
		viFindNext(fList, desc);
	}
}


void setMode(int mode)
{
	SetCtrlVal(panelHandle, PANEL_CHECK_DCV, 0);
	SetCtrlVal(panelHandle, PANEL_CHECK_ACV, 0);
	SetCtrlVal(panelHandle, PANEL_CHECK_ACDC, 0);
	SetCtrlVal(panelHandle, PANEL_CHECK_2WOHM, 0);
	SetCtrlVal(panelHandle, PANEL_CHECK_4WOHM, 0);
	if(connected)
	{	
		CmtGetLock(GPIBLock);
		switch(mode)
		{
			case 0:
				viWrite(instr, "F1\n", 3, &retCount);
				SetCtrlVal(panelHandle, PANEL_CHECK_DCV, 1); 
				break;
			case 1:
				viWrite(instr, "F2\n", 3, &retCount);
				SetCtrlVal(panelHandle, PANEL_CHECK_ACV, 1);
				break;
			case 2:
				viWrite(instr, "F3\n", 3, &retCount);
				SetCtrlVal(panelHandle, PANEL_CHECK_ACDC, 1);
				break;
			case 3:
				viWrite(instr, "F4\n", 3, &retCount);
				SetCtrlVal(panelHandle, PANEL_CHECK_2WOHM, 1); 
				break;
			case 4:
				viWrite(instr, "F5\n", 3, &retCount);
				SetCtrlVal(panelHandle, PANEL_CHECK_4WOHM, 1);
				break;
			default:
				break;
		}
		CmtReleaseLock(GPIBLock);
	}
}

/* Non-zero means error */
int init3456()
{
	if(viWrite(instr, "F1\n", 3, &retCount) < VI_SUCCESS)return -1; /* DCV mode */
	viWrite(instr, "R1\n", 3, &retCount); /* 100mV range */
	viWrite(instr, "T1\n", 3, &retCount); /* Single trigger */
	viWrite(instr, "Z1\n", 3, &retCount); /* Auto zero */
	
	viWrite(instr, "6STG\n", 5, &retCount); /* 6.5 Digit display mode */
	viWrite(instr, "100STI\n", 6, &retCount); /* 10 pts average */
	
	setMode(0);
	
	return 0;
}


int CVICALLBACK acqThread(void *functionData)
{
	double value = 0.0f;
	double currentTime = 0.0f;
	
	SetBreakOnLibraryErrors (0);
    while (!quit)
	{
		if(connected)
		{
			
			SetCtrlVal(panelHandle, PANEL_LED_TRIGGER, 0); 
			
			CmtGetLock(GPIBLock);
			if(viRead(instr, buffer, 200, &retCount) < VI_SUCCESS)
			{
				MessagePopup("Error", "Unable to fetch data from 3456...");
				connected = 0;
				SetCtrlAttribute(panelHandle, PANEL_BUTTON_OPENGPIB, ATTR_LABEL_TEXT, "Open"); 	
				SetCtrlAttribute(panelHandle, PANEL_LED_CONNECTED, ATTR_CTRL_VAL, 0);
				viClose(instr);
			}
			CmtReleaseLock(GPIBLock);
			
			pointCounter++;
			SetCtrlVal(panelHandle, PANEL_LED_TRIGGER, 1); 
			sscanf(buffer, "%lf", &value);
			SetCtrlVal(panelHandle, PANEL_NUMERIC_READ, value); 
			PlotStripChartPoint(panelHandle,  PANEL_CHART_TIME, value);
			GetCurrentDateTime(&currentTime);
			SetCtrlAttribute(panelHandle, PANEL_CHART_TIME, ATTR_XAXIS_GAIN, currentTime - startTime);
		}
		Sleep(100);
    }
    return 0;
}

/// HIFN The main entry-point function.
int main (int argc, char *argv[])
{
    int error = 0;
	int functionID = 0;
    
	SetBreakOnLibraryErrors (0);
    /* initialize and load resources */
    nullChk (InitCVIRTE (0, argv, 0));
    errChk (panelHandle = LoadPanel (0, "3456Terminal.uir", PANEL));
    SetAxisRange(panelHandle, PANEL_CHART_TIME, VAL_AUTOSCALE, 0, 0, VAL_AUTOSCALE, 0, 0);
		
	/* Open VISA */
	if (viOpenDefaultRM(&defaultRM) < VI_SUCCESS) 
	{
		MessagePopup("Error", "Can't open VISA resource");
		return 0;
	}
	enumInstruments();
	
	CmtNewLock(NULL, 0, &GPIBLock);
	/* Create acquisition thread for reading HP3456 */
	CmtScheduleThreadPoolFunction (DEFAULT_THREAD_POOL_HANDLE, acqThread, NULL, &functionID);
 
    /* display the panel and run the user interface */
    errChk (DisplayPanel (panelHandle));
    errChk (RunUserInterface ());


Error:
    /* clean up */
    DiscardPanel (panelHandle);
	
	quit = 1;
	CmtWaitForThreadPoolFunctionCompletion (DEFAULT_THREAD_POOL_HANDLE, functionID, 0);
	CmtDiscardLock(GPIBLock);
    return 0;
}

//==============================================================================
// UI callback function prototypes

/// HIFN Exit when the user dismisses the panel.
int CVICALLBACK panelCB (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    if (event == EVENT_CLOSE)
	{
		viClose(instr);
		viClose(defaultRM);
        QuitUserInterface (0);
	}
    return 0;
}

int CVICALLBACK open_gpib (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	ViChar deviceName[32]; 
	switch (event)
	{
		case EVENT_COMMIT:
			
			if(connected)
			{
				connected = 0;
				SetCtrlAttribute(panel, PANEL_BUTTON_OPENGPIB, ATTR_LABEL_TEXT, "Open"); 	
				SetCtrlAttribute(panel, PANEL_LED_CONNECTED, ATTR_CTRL_VAL, 0);
				viClose(instr);
			}
			else
			{
				GetCtrlVal(panelHandle, PANEL_MENU_DEVICE, deviceName);
				if(viOpen(defaultRM, deviceName, VI_NULL, VI_NULL, &instr) < VI_SUCCESS)
				{
					MessagePopup("Error", "Can't open GPIB");
					return 0;
				}
				viSetAttribute(instr, VI_ATTR_TMO_VALUE, 10000); /* Set timeout */

				if(init3456())
				{
					MessagePopup("Error", "Can't connect to HP3456");
					return 0;	
				}
			
				connected = 1;
				SetCtrlAttribute(panel, PANEL_BUTTON_OPENGPIB, ATTR_LABEL_TEXT, "Close"); 
				SetCtrlAttribute(panel, PANEL_LED_CONNECTED, ATTR_CTRL_VAL, 1);
				GetCurrentDateTime(&startTime);
				SetCtrlAttribute(panelHandle, PANEL_CHART_TIME, ATTR_XAXIS_OFFSET, startTime);
			}
			break;
	}
	return 0;
}

int CVICALLBACK scan_gpib (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			enumInstruments();
			break;
	}
	return 0;
}

int CVICALLBACK clear_graph (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			ClearStripChart(panelHandle, PANEL_CHART_TIME);
			GetCurrentDateTime(&startTime);
			SetCtrlAttribute(panelHandle, PANEL_CHART_TIME, ATTR_XAXIS_OFFSET, startTime);
			break;
	}
	return 0;
}

int CVICALLBACK DCV_check (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			setMode(0);
			break;
	}
	return 0;
}

int CVICALLBACK ACV_check (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			setMode(1);
			break;
	}
	return 0;
}

int CVICALLBACK ACDC_check (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			setMode(2);
			break;
	}
	return 0;
}

int CVICALLBACK Ohm2W_check (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			setMode(3);
			break;
	}
	return 0;
}

int CVICALLBACK Ohm4W_check (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			setMode(4);
			break;
	}
	return 0;
}
